package bezier;

import java.lang.*;
import java.math.*;
import java.awt.*;

/**
 * ٥
 */
public class BCurve{
    public Vec[] control_points;

    public BCurve(Vec[] points){
        if(points.length == 4){
            control_points = points;
        }
        else{
            System.err.println("Needs 4 control points");
        }
    }

    public BCurve(Vec p1, Vec p2, Vec p3, Vec p4){
        control_points = new Vec[4];
        control_points[0] = p1;
        control_points[1] = p2;
        control_points[2] = p3;
        control_points[3] = p4;
    }
    
    public BCurve(){
        control_points = new Vec[4];
        for(int i = 0; i < 4; i++){
            control_points[i] = new Vec();
        }
    }

    public void setControlPoint(int num, Vec val){
        control_points[num] = val;
    }


    /**
     * de CasteljouΥ르ꥺȤät=rζ֤.
     */
    public Vec getPointByCasteljou(double r){
        double p0_x = control_points[0].x;
        double p0_y = control_points[0].y;
        double p0_z = control_points[0].z;
        double p1_x = control_points[1].x;
        double p1_y = control_points[1].y;
        double p1_z = control_points[1].z;
        double p2_x = control_points[2].x;
        double p2_y = control_points[2].y;
        double p2_z = control_points[2].z;
        double p3_x = control_points[3].x;
        double p3_y = control_points[3].y;
        double p3_z = control_points[3].z;

        double p10_x = p0_x+r*(p1_x-p0_x);
        double p10_y = p0_y+r*(p1_y-p0_y);
        double p10_z = p0_z+r*(p1_z-p0_z);
        
        double p11_x = p1_x+r*(p2_x-p1_x);
        double p11_y = p1_y+r*(p2_y-p1_y);
        double p11_z = p1_z+r*(p2_z-p1_z);
        
        double p12_x = p2_x+r*(p3_x-p2_x);
        double p12_y = p2_y+r*(p3_y-p2_y);
        double p12_z = p2_z+r*(p3_z-p2_z);
    
        double p20_x = p10_x+r*(p11_x-p10_x);
        double p20_y = p10_y+r*(p11_y-p10_y);
        double p20_z = p10_z+r*(p11_z-p10_z);

        double p21_x = p11_x+r*(p12_x-p11_x);
        double p21_y = p11_y+r*(p12_y-p11_y);
        double p21_z = p11_z+r*(p12_z-p11_z);
        
        double p30_x = p20_x+r*(p21_x-p20_x);
        double p30_y = p20_y+r*(p21_y-p20_y);
        double p30_z = p20_z+r*(p21_z-p20_z);
        
        return new Vec(p30_x, p30_y, p30_z);
    }
    
    public void draw(Graphics g, double d, 
                     Vec fp, double R, double the, double phi){
        double st = Math.sin(the);
        double ct = Math.cos(the);
        double sp = Math.sin(phi);
        double cp = Math.cos(phi);

        draw(g, d, fp, R, st, ct, sp, cp);
    }

    public void drawCP(Graphics g, Vec fp, double R, double the, double phi){
        for(int i = 0; i < 4; i++){
            control_points[i].calcPerspectiveProjection(fp,R,the,phi);
            g.fillRect(((int)control_points[i].px) - 1, 
                       (-(int)control_points[i].py) - 1, 3, 3);
        }
         
        Vec s;
        Vec e;
        for(int i = 0; i < 3; i++){
            s = control_points[i];
            e = control_points[i+1];
            g.drawLine((int)s.px, -(int)s.py, (int)e.px, -(int)e.py);
        }
        
    }

    public void draw(Graphics g, double d, 
                     Vec fp, double R, 
                     double sin_the, double cos_the,
                     double sin_phi, double cos_phi){
     
        double t = 0.0;
        Vec s = getPointByCasteljou(t);
        s.calcPerspectiveProjection(fp, R, sin_the, cos_the, sin_phi, cos_the);
        t += d;
        Vec n;
        while(t < 1){
            n = getPointByCasteljou(t);
            n.calcPerspectiveProjection(fp, R, sin_the, cos_the, sin_phi, cos_the);
            g.drawLine((int)s.px, -(int)s.py, (int)n.px, -(int)n.py);
            s = n;
            t += d;
        }
        n = getPointByCasteljou(1.0);
        n.calcPerspectiveProjection(fp, R, sin_the, cos_the, sin_phi, cos_the);
        g.drawLine((int)s.px, -(int)s.py, (int)n.px, -(int)n.py);    
    }

    public static void main(String[] argv){

        BCanvas canv = new BCanvas(400,400);
        Frame f = new Frame("Bezier");
        f.setLayout(null);
        f.setBackground(Color.white);
        f.setSize(500,500);
        f.move(100, 100);

        canv.setBackground(Color.black);
        canv.move(10, 10);
        f.add(canv);

        f.setVisible(true);
        canv.setVisible(true);
        canv.makeOffScreen();

        Graphics offsg = canv.getOffScreenGraphics();
        offsg.translate(0, canv.size().height);

        Vec[] cps = new Vec[4];
        cps[0] = new Vec(10.0, 30.0, 0.0);
        cps[1] = new Vec(100.0, 280.0, 0.0);
        cps[2] = new Vec(200.0, 300.0, 0.0);
        cps[3] = new Vec(350.0, 100.0, 0.0);
        
        BCurve bc = new BCurve(cps);
        

        //offsg.fillRect(20, 20, 40, 40);
        //offsg.drawLine(1, 1, 100, 100);

        
        Vec fp = new Vec(350.0, 350.0, 0.1);
        double R = 100000.0;

        double the = Math.PI/2.0;
        double phi = Math.PI/2.0;

        offsg.setColor(Color.red);
        bc.drawCP(offsg, fp, R, the, phi);
        offsg.setColor(Color.white);
        bc.draw(offsg, 0.02, fp, R, the, phi);

        
        while(true){
            int i = 0;
        }
  
    }
     
}




