package bezier;

import java.lang.*;
import java.util.*;
import java.awt.*;

public class BSurface{

    public Vec origin = new Vec(0.0, 0.0, 0.0);
    public Vec xaxis  = new Vec(1000.0, 0.0, 0.0);
    public Vec yaxis  = new Vec(0.0, 1000.0, 0.0);
    public Vec zaxis  = new Vec(0.0, 0.0, 1000.0);

    public Vec[][] control_points; // 4 x 4
    public Vec[][] points;

    public BSurface(Vec[][] cps){
        this.control_points = cps;
    }


    public BSurface(double xw, double yw, double zh){
        Random rand = new Random();
        double h;
        control_points = new Vec[4][4];
        for(int i = 0; i < 4; i++){
            for(int j = 0; j < 4; j++){
                h = rand.nextDouble() * zh;
                control_points[i][j] = new Vec((i*xw)/4.0, (j*yw)/4.0, h);
            }
        }                                        
    }

    public BSurface(){
        control_points = new Vec[4][4];
        for(int i = 0; i < 4; i++){
            for(int j = 0; j < 4; j++){
                control_points[i][j] = new Vec();
            }
        }
    }

    public void setControlPoint(int i, int j, Vec val){
        control_points[i][j] = val;
    }

    /**
     * u,v򤽤줾 u_num, v_num v=const
     */
    public void calcPoints(int u_num, int v_num){
        points = new Vec[u_num][v_num];
        
        double du = 1.0/(u_num-1);
        double dv = 1.0/(v_num-1);

        BCurve[] col_curve = new BCurve[4];
        for(int i = 0; i < 4; i++){
            col_curve[i] = new BCurve(control_points[i][0],
                                      control_points[i][1],
                                      control_points[i][2],
                                      control_points[i][3]);
        }

        double v = 0.0;
        double u = 0.0;

        Vec[]  v_const_curve_cps = new Vec[4];
        BCurve v_const_curve;

        for(int j = 0; j < v_num; j++){
            v_const_curve_cps[0] = col_curve[0].getPointByCasteljou(v);
            v_const_curve_cps[1] = col_curve[1].getPointByCasteljou(v);
            v_const_curve_cps[2] = col_curve[2].getPointByCasteljou(v);
            v_const_curve_cps[3] = col_curve[3].getPointByCasteljou(v);
            v_const_curve = new BCurve(v_const_curve_cps);
            u = 0.0;
            for(int i = 0; i < u_num; i++){
                points[i][j] = v_const_curve.getPointByCasteljou(u);
                u += du;
            }
            v += dv;
        }            
    }

    public void draw(Graphics g,
                     Vec fp, double R, double the, double phi){
        double st = Math.sin(the);
        double ct = Math.cos(the);
        double sp = Math.sin(phi);
        double cp = Math.cos(phi);

        draw(g, fp, R, st, ct, sp, cp);
    }

    public void draw(Graphics g,
                     Vec fp, double R, 
                     double sin_the, double cos_the,
                     double sin_phi, double cos_phi){
       
        if(points == null){
            return;
        }
        int jl = points[0].length;
        int il = points.length;
        for(int j = 0; j < jl; j++){
            for(int i = 0; i < il; i++){
                points[i][j].calcPerspectiveProjection(fp, R, 
                                                       sin_the, cos_the,
                                                       sin_phi, cos_phi);
            }
        }
    
        Vec s;
        Vec e;
        for(int j = 0; j < jl; j++){
            for(int i = 0; i < il-1; i++){
                s = points[i][j];
                e = points[i+1][j];
                if(s.visible && e.visible){
                    g.drawLine((int)s.px, -(int)s.py, (int)e.px, -(int)e.py);
                }
            }
        }
    }

    
    public void drawAxis(Graphics g,
                         Vec fp, double R,
                         double the, double phi){
        double st = Math.sin(the);
        double ct = Math.cos(the);
        double sp = Math.sin(phi);
        double cp = Math.cos(phi);

        drawAxis(g, fp, R, st, ct, sp, cp);
    }

    public void drawAxis(Graphics g,
                         Vec fp, double R, 
                         double sin_the, double cos_the,
                         double sin_phi, double cos_phi){
        origin.calcPerspectiveProjection(fp,R,sin_the,cos_the,sin_phi,cos_phi);
        xaxis.calcPerspectiveProjection(fp,R,sin_the,cos_the,sin_phi,cos_phi);
        yaxis.calcPerspectiveProjection(fp,R,sin_the,cos_the,sin_phi,cos_phi);
        zaxis.calcPerspectiveProjection(fp,R,sin_the,cos_the,sin_phi,cos_phi);
        if(origin.visible){
            if(xaxis.visible){
                g.drawLine((int)origin.px,-(int)origin.py,(int)xaxis.px,-(int)xaxis.py);
            }
            if(yaxis.visible){
                g.drawLine((int)origin.px,-(int)origin.py,(int)yaxis.px,-(int)yaxis.py) ;
            }
            if(zaxis.visible){
                g.drawLine((int)origin.px,-(int)origin.py,(int)zaxis.px,-(int)zaxis.py) ;
            }
        }
    }

    public void drawCP(Graphics g,
                       Vec fp, double R,
                       double the, double phi){
        double st = Math.sin(the);
        double ct = Math.cos(the);
        double sp = Math.sin(phi);
        double cp = Math.cos(phi);

        drawCP(g, fp, R, st, ct, sp, cp);
    }

    public void drawCP(Graphics g,
                       Vec fp, double R, 
                       double sin_the, double cos_the,
                       double sin_phi, double cos_phi){
        for(int i = 0; i < 4; i++){
            for(int j = 0; j < 4; j++){
                control_points[i][j].calcPerspectiveProjection(fp, R, sin_the, cos_the, sin_phi, cos_phi);
                if(control_points[i][j].visible){
                    g.fillRect(((int)control_points[i][j].px) - 1, 
                               (-(int)control_points[i][j].py) - 1, 3, 3);
                }
            }
        }
    }


    public void drawHide(Graphics g,
                         Vec fp, double R,
                         double the, double phi){
        double st = Math.sin(the);
        double ct = Math.cos(the);
        double sp = Math.sin(phi);
        double cp = Math.cos(phi);

        drawHide(g, fp, R, st, ct, sp, cp);
    }                

    public void drawHide(Graphics g,
                         Vec fp, double R, 
                         double sin_the, double cos_the,
                         double sin_phi, double cos_phi){
      
        System.err.println("drawHide");
        if(points == null){
            return;
        }
        int jl = points[0].length;
        int il = points.length;
        for(int j = 0; j < jl; j++){
            for(int i = 0; i < il; i++){
                points[i][j].calcPerspectiveProjection(fp, R, 
                                                       sin_the, cos_the,
                                                       sin_phi, cos_phi);
            }
        }
    
        int px_min = (int)points[0][0].px;
        int px_max = (int)points[0][0].px;

        int py_min = (int)points[0][0].py;
        int py_max = (int)points[0][0].py;

        Vec n;
        for(int j = 0; j < jl; j++){
            for(int i = 0; i < il; i++){
                n = points[i][j];
                if((int)n.px > px_max){ px_max = (int)n.px; }
                if((int)n.px < px_min){ px_min = (int)n.px; }
                if((int)n.py > py_max){ py_max = (int)n.py; }
                if((int)n.py < py_min){ py_min = (int)n.py; }
            }
        }

        int[] min_buffer = new int[px_max-px_min+1];
        int[] max_buffer = new int[px_max-px_min+1];
        
       
        Vec s;
        Vec e;
       
        
        for(int i = 0; i < il-1; i++){
            s = points[i][0];
            e = points[i+1][0];
            double slope = (e.py-s.py)/(e.px-s.px);
            
            for(int k = (int)s.px; k <= e.px; k++){
                min_buffer[k-px_min] = (int)(s.py + slope*(k-(int)s.px));
                max_buffer[k-px_min] = (int)(s.py + slope*(k-(int)s.px));
            }
        }

        
        for(int j = 0; j < jl; j++){
            for(int i = 0; i < il-1; i++){
                s = points[i][j];
                e = points[i+1][j];
                if(s.visible && e.visible){
                    double slope = (e.py-s.py)/(e.px-s.px);
                    int y;
                    for(int k = (int)s.px; k <= e.px; k++){
                        y = (int)(s.py+slope*(k-(int)s.px));
                        if(y >= max_buffer[k-px_min]){
                            max_buffer[k-px_min] = y;
                            g.drawLine(k, -y, k, -y );
                        }
                        if(y <= min_buffer[k-px_min]){
                            min_buffer[k-px_min] = y;
                            g.drawLine(k, -y, k, -y);
                        }
                    }
                }
            }
        }
    }

    public static void main(String[] argv){

        BCanvas canv = new BCanvas(400,400);
        Frame f = new Frame("Bezier");
        f.setLayout(null);
        f.setBackground(Color.white);
        f.setSize(500,500);
        f.move(100, 100);

        canv.setBackground(Color.black);
        canv.move(10, 10);
        f.add(canv);

        f.setVisible(true);
        canv.setVisible(true);
        canv.makeOffScreen();

        
        Graphics offsg = canv.getOffScreenGraphics();
        offsg.translate((int)(canv.size().width/2.0), 
                        (int)(canv.size().height/2.0));

              

        //offsg.fillRect(20, 20, 40, 40);
        //offsg.drawLine(1, 1, 100, 100);

        BSurface bs = new BSurface(350, 350, 400);
        bs.calcPoints(200, 50);

        Vec fp = new Vec(200, -1.0, 200);
        
        double R = 100000.0;

        double the = Math.PI/4.0*3.0;
        double phi = Math.PI;

        offsg.setColor(Color.red);
        bs.drawCP(offsg, fp, R, the, phi);
        offsg.setColor(Color.white);
        bs.draw(offsg, fp, R, the, phi);

        while(true){
            int i = 0;
        }
  
    }
    
}
