package bezier;

import java.awt.*;
import java.awt.event.*;

public class BezierApp implements MouseMotionListener, MouseListener{
    public static double TPI = Math.PI * 2.0;

    public static int NOT_HIDE = 0;
    public static int HIDE     = 1;

    public static double RD = 100;
    public static double THE_D = Math.PI/40.0;
    public static double PHI_D = Math.PI/40.0;
    
    public static Vec    init_fp  = new Vec(170, 300, 180);
    public static double init_R   = 8000.0;
    public static double init_the = Math.PI/2.0 * 3.0;
    public static double init_phi = 0.0;

    public static int DIV_U = 500;
    public static int DIV_V = 50;

    BCanvas canv;
    Frame   f;
    
    Button  hide_button;
    Button  zi_button;
    Button  zo_button;
    Button  cl_button;

    BSurface bs;
    
    Vec fp     = init_fp;
    double R   = init_R;
    double the = init_the;
    double phi = init_phi;

    int md_x;
    int md_y;

    int hide_state = NOT_HIDE;

    public BezierApp(){
        canv   = new BCanvas(400, 400);
        f      = new Frame("Bezier");
        f.setLayout(null);
        //f.setBackground(Color.white);
        f.setSize(440,480);

        f.move(100, 100);

        canv.setBackground(Color.black);
        canv.move(10, 10);
        f.add(canv);

        f.setVisible(true);
        canv.setVisible(true);
        
        hide_button = new Button("hide");
        zi_button   = new Button("Zoom in");
        zo_button   = new Button("Zoom out");
        cl_button   = new Button("clear");
        
        hide_button.move(20, 430);
        hide_button.setSize(50, 20);
        hide_button.setVisible(true);
        f.add(hide_button);

        zi_button.move(100, 430);
        zi_button.setSize(50, 20);
        zi_button.setVisible(true);
        f.add(zi_button);

        zo_button.move(200, 430);
        zo_button.setSize(50, 20);
        zo_button.setVisible(true);
        f.add(zo_button);

        cl_button.move(300, 430);
        cl_button.setSize(60, 20);
        cl_button.setVisible(true);
        f.add(cl_button);
        
        canv.makeOffScreen();
        
        bs = new BSurface(300, 300, 300);
        bs.calcPoints(DIV_U, DIV_V);
        
        canv.addMouseMotionListener(this);
        canv.addMouseListener(this);
        hide_button.addMouseListener(this);
        zi_button.addMouseListener(this);
        zo_button.addMouseListener(this);
        cl_button.addMouseListener(this);
        f.addWindowListener(new WindowAdapter(){
            public void windowClosing(WindowEvent ev){
                System.exit(0);
            }
        });
        draw();
    }

    public void mouseEntered(MouseEvent me){}
    public void mouseExited(MouseEvent me){}
    public void mouseReleased(MouseEvent me){}


    public void mousePressed(MouseEvent me){
        Component c = me.getComponent();
        if(c == canv){
            md_x = me.getX();
            md_y = me.getY();
        }
    }

    public void mouseClicked(MouseEvent me){
        Component c = me.getComponent();
        if(c == hide_button){
            if(hide_state == NOT_HIDE){
                hide_state = HIDE;
                hide_button.setLabel("show");
            }
            else{
                hide_state = NOT_HIDE;
                hide_button.setLabel("hide");
            }
            draw();
        }
        else if(c == zi_button){
            R -= RD;
            draw();
        }
        else if(c == zo_button){
            R += RD;
            draw();
        }
        else if(c == cl_button){
            reset();
            draw();
        }
    }

    public void mouseMoved(MouseEvent me){}

    public void mouseDragged(MouseEvent me){

        if(me.getComponent() != canv){
            return;
        }
        int x = me.getX();
        int y = me.getY();

        int dx = (x-md_x);
        int dy = (y-md_y);
        int dxx = dx*dx;
        int dyy = dy*dy;

        if(dxx > dyy){
            if(dx < 0){
                the -= THE_D;
            }
            else{
                the += THE_D;
            }
        }
        else{
            if(dy < 0){
                phi -= PHI_D;
            }
            else{
                phi += PHI_D;
            }
        }

        if(the < 0){
            the += TPI;
        }
        if(the > TPI){
            the -= TPI;
        }
        
        if(phi < 0){
            phi += TPI;
        }
        if(phi > TPI){
            phi -= TPI;
        }

        draw();
        md_x = x;
        md_y = y;
    }

    public void draw(){
        canv.clearOffScreen();

        Graphics offsg = canv.getOffScreenGraphics();
        offsg.translate((int)(canv.size().width/2.0), 
                        (int)(canv.size().height/2.0));

        offsg.setColor(Color.blue);
        bs.drawAxis(offsg, fp, R, the, phi);

        offsg.setColor(Color.red);
        bs.drawCP(offsg, fp, R, the, phi);
        offsg.setColor(Color.white);
        if(hide_state == HIDE){
            bs.drawHide(offsg, fp, R, the, phi);
        }
        else{
            bs.draw(offsg, fp, R, the, phi);
        }

        String info1 = "fp = (" + fp.x + "," + fp.y + "," + fp.z +")";
        String info2 = "R = " + (float)R + " ";
        info2 += "the = " + (float)the + " ";
        info2 += "phi = " + (float)phi;
        
        Graphics g = canv.getOffScreenGraphics();
        g.setFont(new Font("Serif", Font.PLAIN, 12));
        g.drawString(info1, 8, 30);
        g.drawString(info2, 8, 48);
        canv.paint(canv.getGraphics());
        
    }

    public void reset(){
        bs = new BSurface(380, 380, 400);
        R = init_R;
        the = init_the;
        phi = init_phi;
        bs.calcPoints(DIV_U, DIV_V);
        draw();
    }

    public static void main(String[] argv){
        BezierApp ba = new BezierApp();
    } 
}

