package bezier;

public class Vec{  
    public static final double TPI = Math.PI * 2.0;

    public double x,y,z;
  
    /* $B=i4|CM(B */
    public double ox,oy,oz; 
    
    /* $B=i4|>uBV$+$i$I$l$[$I2sE>$7$?$+(B */
    public double rot_x,rot_y,rot_z;
    
    /* xy$BJ?LL$KJ?9T$JJ?LL(B(z = vp)$B$KF);kEj1F$7$?$H$-$N:BI8$r5-21(B */
    /* $B;kE@$O(Bz$B<4>e(B( > vp )                   */
    public double px, py, pz;
    public boolean visible;

    public Vec(double x, double y, double z){
        this.x = x; this.y = y; this.z = z;
        ox = x; oy = y; oz = z;
        rot_x = 0.0; rot_y = 0.0; rot_z = 0.0;
    }

    public Vec(){
        this.x = 0.0; this.y = 0.0; this.z = 0.0;
        ox = x; oy = y; oz = z;
        rot_x = 0.0; rot_y = 0.0; rot_z = 0.0;
    }

    public void returnToInit(){
        x = ox; y = oy; z = oz;
        rot_x = 0.0; rot_y = 0.0; rot_z = 0.0;
    }
    
    public void setCurrentAsInit(){
        ox = x; oy = y; oz = z;
        rot_x = 0.0; rot_y = 0.0; rot_z = 0.0;
    }
    
    public void translate(double tx, double ty, double tz){
        x += tx;
        y += ty;
        z += tz;
    }
    
    public void xAxisRotate(double r){
        rot_x += r;
        if(rot_x > TPI){
            rot_x -= TPI;
        }
        else if( rot_x < - TPI){
            rot_x += TPI;
        }
        rotateFromInit();
    }
    
    public void yAxisRotate(double r){
        rot_y += r;
        if(rot_y > TPI){
            rot_y -= TPI;
        }
        else if( rot_y < - TPI){
            rot_y += TPI;
        }
        rotateFromInit();
        
    }
    
    public void zAxisRotate(double r){
        rot_z += r;
        if(rot_z > TPI){
            rot_z -= TPI;
        }
        else if( rot_z < - TPI){
            rot_z += TPI;
        }
        rotateFromInit();
    }
    
    public void rotateFromInit(){
        double cosr = Math.cos(rot_x);
        double sinr = Math.sin(rot_x);
        double tx,ty,tz;
        
        x = ox;
        y = oy;
        z = oz;
        
        ty = y;
        tz = z;
        y = ty*cosr - tz*sinr;
        z = ty*sinr + tz*cosr;
        
        cosr = Math.cos(rot_y);
        sinr = Math.sin(rot_y);
        
        tz = z;
        tx = x;
        x = tz*sinr + tx*cosr;
        z = tz*cosr - tx*sinr;
        
        cosr = Math.cos(rot_z);
        sinr = Math.sin(rot_z);
        
        tx = x;
        ty = y;
        x = tx*cosr - ty*sinr;
        y = tx*sinr + ty*cosr;
        
    }
    
    /* $B;kE@$r(B (0,0,prp) prp > vp $B$K$*$$$F(B xy$BJ?LL$KJ?9T$JJ?LL(B(z = vp)
       $B$KF);kEj1F$7$?$H$-$N:BI8$r7W;;$7$F%;%C%H$9$k(B  */
    public void calcPerspectiveProjection(double vp, double prp){
        if( z >= vp ){
            visible = false;
        }
        else{
            visible = true;
        }
        
        double d = (prp - vp)/(prp - z);
        px = x*d;  
        py = y*d;
        pz = z*d;
    }
 
    /**
     * $BCm;kE@(B(fp)$B$r4^$`J?LL$X$NEj;qEj1F$7$?$H$-$N:BI8$r(B
     * $B7W;;$7$F%;%C%H$9$k!#;kE@$O(Bfp$B$+$i$N5wN%(BR, $BJ}0L3Q(Btheta, $B6D3Q(Bphi
     * $B$G<($9!#(B
     */
    public void calcPerspectiveProjection(Vec fp, 
                                          double R, double the, double phi){
        double st = Math.sin(the);
        double ct = Math.cos(the);
        double sp = Math.sin(phi);
        double cp = Math.cos(phi);
        calcPerspectiveProjection(fp, R, st, ct, sp, cp);
       
    }

    /**
     * $BCm;kE@(B(fp)$B$r4^$`J?LL$X$NEj;qEj1F$7$?$H$-$N:BI8$r(B
     * $B7W;;$7$F%;%C%H$9$k!#;kE@$O(Bfp$B$+$i$N5wN%(BR, $BJ}0L3Q(Btheta, $B6D3Q(Bphi
     * $B$G<($9!#(B
     */
    public void calcPerspectiveProjection(Vec fp, 
                                          double R,
                                          double sin_the,
                                          double cos_the,
                                          double sin_phi,
                                          double cos_phi){
        
        double xf = fp.x;
        double yf = fp.y;
        double zf = fp.z;
        double xd = (x - xf);
        double yd = (y - yf);
        double zd = (z - zf);
        double st = sin_the;
        double ct = cos_the;
        double sp = sin_phi;
        double cp = cos_phi;
        double xx = -xd*st+yd*ct;
        double yy = -xd*ct*sp-yd*st*sp+zd*cp;
        double zz = xd*ct*cp+yd*st*cp+zd*sp;
        
        if(zz >= R){
            visible = false;
        }
        else{
            visible = true;
        }
        double d = R/(R-zz);
        px = xx*d;
        py = yy*d;
        pz = zz*d;
        
    }

    public void calcParallelProjection(double vp){
        if( z >= vp ){
            visible = false;
        }
        else{
            visible = true;
        }
        px = x;
        py = y;
        pz = z;
    }
    
    public boolean in(int wind_minx, int wind_miny, 
                      int wind_maxx, int wind_maxy){
        return (   (px >= wind_minx) && (px <= wind_maxx)
                && (py >= wind_miny) && (py <= wind_maxy));
    }
    
}





